/*****************************************************************************/
/* trim_wtmp_main.c                                                          */
/*                                                                           */
/* Reads an utmp/wtmp like file from stdin and copies it to stdout, filter-  */
/* tering the output according to supplied criterias.                        */
/*                                                                           */
/* (c) 1995 by Markus Schoepflin. See README for details.                    */
/* Modified 30 December 1998 by Brian Huntley@sunlife.com for Y2K (YYYY)     */
/* Modified 22 March 1999 by Brian Huntley for -b option                     */
/*****************************************************************************/

#include "trim_wtmp_func.h"

#ifndef LINUX

extern char *optarg;

#else

#include <getopt.h>

#endif

char *pname;

/* Parses the string 'date' for a date in the form YYYY-MM-DD and returns the
   corresponding time_t value or (time_t)-1 on error. (Same as 'mktime')
*/

void usage(const char* pname) {

  fprintf(stderr, "Usage: %s { -s YYYY-MM-DD | -b YYYY-MM-DD | -d days } [-v]\n", pname);
  exit(1);

}

time_t scandate(const char *date) {

  struct tm tm_tmp;

				/* the format assumed is YYYY-MM-DD */
  sscanf(date, "%d-%d-%d",
	 &(tm_tmp.tm_year), &(tm_tmp.tm_mon), &(tm_tmp.tm_mday));
  
  tm_tmp.tm_mon--;		/* no. of months since january */

  if (tm_tmp.tm_year < 1900) usage(pname); /* confirm valid date */
  tm_tmp.tm_year  -= 1900;	/* no. of years since 1900 */
  tm_tmp.tm_sec   =  0;		/* init the rest of the tm struct */
  tm_tmp.tm_min   =  0;
  tm_tmp.tm_hour  =  0;
  tm_tmp.tm_isdst = -1;		/* let mktime find out */

  return mktime(&tm_tmp);

}

time_t calcdate(const char *days) {

#include <sys/time.h>

  time_t my_time;
  int	 idays;

  sscanf(days, "%d", &(idays));
  my_time = time(0) - (idays*24*60*60);
  return my_time;

}

main(int argc, char* argv[]) {

  BOOL verbose = FALSE;
  BOOL gotdate = FALSE;
  BOOL before  = FALSE;
  S_UTMP *entry;		/* the current utmp/wtmp entry */
  time_t t_cut;			/* the cutoff time */
  u_int c;

  pname = argv[0];

  while ((c = getopt(argc, argv, "s:b:d:v")) != -1) {

    switch (c) {

    case 's':			/* optarg is explicit date */
      gotdate = TRUE;
      before = FALSE;
      t_cut = scandate(optarg);
      break;

    case 'b':			/* optarg is explicit date */
      gotdate = TRUE;
      before = TRUE;
      t_cut = scandate(optarg);
      break;

    case 'd':			/* optarg is number of days in past */
      gotdate = TRUE;
      before = FALSE;
      t_cut = calcdate(optarg);
      break;

    case 'v':			/* be verbose */
      verbose = TRUE;
      break;

    default:
      usage(pname);
      break;

    }

  }

  if (!gotdate) usage(pname);

  if (verbose) {
    if (before) {
        fprintf(stderr, "\nReporting entries before and including %s\n",
            ctime(&t_cut));
    } else {
        fprintf(stderr, "\nReporting entries after and including %s\n",
            ctime(&t_cut));
    }
    my_printheader(stderr);
  }

  while (entry = my_getutent(stdin)) {

    if (my_selectutent(entry, t_cut, before)) {
      if (verbose)
	my_printutent(entry, stderr);
      my_pututent(stdout);
    }
  }
  exit(0);
}
